package com.mdd.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mdd.admin.config.quartz.QuartzUtils;
import com.mdd.admin.service.ICrontabService;
import com.mdd.admin.validate.commons.PageValidate;
import com.mdd.admin.validate.crontab.CrontabCreateValidate;
import com.mdd.admin.validate.crontab.CrontabUpdateValidate;
import com.mdd.admin.vo.CrontabDetailVo;
import com.mdd.admin.vo.CrontabListedVo;
import com.mdd.common.core.PageResult;
import com.mdd.common.entity.Crontab;
import com.mdd.common.mapper.CrontabMapper;
import com.mdd.common.util.TimeUtils;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

@Service
public class CrontabServiceImpl implements ICrontabService {
   @Resource
   Scheduler scheduler;
   @Resource
   CrontabMapper crontabMapper;

   @PostConstruct
   public void init() throws SchedulerException {
      this.scheduler.clear();
      List<Crontab> jobs = this.crontabMapper.selectList((Wrapper)(new QueryWrapper()).eq("is_delete", 0));
      Iterator var2 = jobs.iterator();

      while(var2.hasNext()) {
         Crontab crontab = (Crontab)var2.next();
         QuartzUtils.createScheduleJob(this.scheduler, crontab);
      }

   }

   public PageResult<CrontabListedVo> list(PageValidate pageValidate) {
      Integer pageNo = pageValidate.getPageNo();
      Integer pageSize = pageValidate.getPageSize();
      IPage<Crontab> iPage = this.crontabMapper.selectPage(new Page((long)pageNo, (long)pageSize), (Wrapper)((QueryWrapper)(new QueryWrapper()).eq("is_delete", 0)).orderByDesc("id"));
      List<CrontabListedVo> list = new LinkedList();
      Iterator var6 = iPage.getRecords().iterator();

      while(var6.hasNext()) {
         Crontab crontab = (Crontab)var6.next();
         CrontabListedVo vo = new CrontabListedVo();
         BeanUtils.copyProperties(crontab, vo);
         vo.setStartTime(crontab.getStartTime() <= 0L ? "-" : TimeUtils.timestampToDate(crontab.getStartTime()));
         vo.setEndTime(crontab.getEndTime() <= 0L ? "-" : TimeUtils.timestampToDate(crontab.getEndTime()));
         list.add(vo);
      }

      return PageResult.iPageHandle(iPage.getTotal(), iPage.getCurrent(), iPage.getSize(), list);
   }

   public CrontabDetailVo detail(Integer id) {
      Crontab crontab = (Crontab)this.crontabMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).eq("id", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(crontab, "数据不存在!");
      CrontabDetailVo vo = new CrontabDetailVo();
      BeanUtils.copyProperties(crontab, vo);
      return vo;
   }

   @Transactional
   public void add(CrontabCreateValidate createValidate) throws SchedulerException {
      Crontab crontab = new Crontab();
      crontab.setName(createValidate.getName());
      crontab.setTypes(createValidate.getTypes());
      crontab.setCommand(createValidate.getCommand());
      crontab.setRules(createValidate.getRules());
      crontab.setStatus(createValidate.getStatus());
      crontab.setRemark(createValidate.getRemark());
      crontab.setStrategy(createValidate.getStrategy());
      crontab.setConcurrent(createValidate.getConcurrent());
      crontab.setCreateTime(System.currentTimeMillis() / 1000L);
      crontab.setUpdateTime(System.currentTimeMillis() / 1000L);
      this.crontabMapper.insert(crontab);
      QuartzUtils.createScheduleJob(this.scheduler, crontab);
   }

   @Transactional
   public void edit(CrontabUpdateValidate updateValidate) throws SchedulerException {
      Crontab crontab = (Crontab)this.crontabMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).eq("id", updateValidate.getId())).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(crontab, "数据不存在!");
      crontab.setName(updateValidate.getName());
      crontab.setCommand(updateValidate.getCommand());
      crontab.setTypes(updateValidate.getTypes());
      crontab.setRules(updateValidate.getRules());
      crontab.setStatus(updateValidate.getStatus());
      crontab.setRemark(updateValidate.getRemark());
      crontab.setStrategy(updateValidate.getStrategy());
      crontab.setConcurrent(updateValidate.getConcurrent());
      crontab.setUpdateTime(System.currentTimeMillis() / 1000L);
      this.crontabMapper.updateById(crontab);
      QuartzUtils.createScheduleJob(this.scheduler, crontab);
   }

   @Transactional
   public void del(Integer id) throws SchedulerException {
      Crontab crontab = (Crontab)this.crontabMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).eq("id", id)).eq("is_delete", 0)).last("limit 1"));
      Assert.notNull(crontab, "数据不存在!");
      crontab.setIsDelete(1);
      crontab.setDeleteTime(System.currentTimeMillis() / 1000L);
      this.crontabMapper.updateById(crontab);
      this.scheduler.deleteJob(QuartzUtils.getJobKey(crontab.getId(), crontab.getTypes()));
   }
}
